@@ -8,7 +8,7 @@
     :copyright: Copyright 2010 by Yuya Nishihara <yuya@tcha.org>.
     :license: BSD, see LICENSE for details.
 """
-import os, re, subprocess
+import errno, os, re, shlex, subprocess
 try:
     from hashlib import sha1
 except ImportError:  # Python<2.5
@@ -56,13 +56,14 @@ def generate_name(self, node, fileformat):
         return fname, os.path.join(self.builder.outdir, fname)
 
 _ARGS_BY_FILEFORMAT = {
+    'eps': '-teps'.split(),
     'png': (),
     'svg': '-tsvg'.split(),
     }
 
 def generate_plantuml_args(self, fileformat):
     if isinstance(self.builder.config.plantuml, basestring):
-        args = [self.builder.config.plantuml]
+        args = shlex.split(self.builder.config.plantuml)
     else:
         args = list(self.builder.config.plantuml)
     args.extend('-pipe -charset utf-8'.split())
@@ -125,7 +126,7 @@ def _get_svg_tag(self, fnames, alt):
         _get_png_tag(self, fnames, alt),
         '</object>'])
 
-_KNOWN_FORMATS = {
+_KNOWN_HTML_FORMATS = {
     'png': (('png',), _get_png_tag),
     'svg': (('png', 'svg'), _get_svg_tag),
     }
@@ -134,11 +135,11 @@ def html_visit_plantuml(self, node):
     try:
         format = self.builder.config.plantuml_output_format
         try:
-            fileformats, gettag = _KNOWN_FORMATS[format]
+            fileformats, gettag = _KNOWN_HTML_FORMATS[format]
         except KeyError:
             raise PlantUmlError(
                 'plantuml_output_format must be one of %s, but is %r'
-                % (', '.join(map(repr, _KNOWN_FORMATS)), format))
+                % (', '.join(map(repr, _KNOWN_HTML_FORMATS)), format))
         # fnames: {fileformat: (refname, outfname), ...}
         fnames = dict((e, render_plantuml(self, node, e))
                       for e in fileformats)
@@ -151,15 +152,65 @@ def html_visit_plantuml(self, node):
     self.body.append('</p>\n')
     raise nodes.SkipNode
 
+def _convert_eps_to_pdf(self, refname, fname):
+    if isinstance(self.builder.config.plantuml_epstopdf, basestring):
+        args = shlex.split(self.builder.config.plantuml_epstopdf)
+    else:
+        args = list(self.builder.config.plantuml_epstopdf)
+    args.append(fname)
+    try:
+        try:
+            p = subprocess.Popen(args, stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+        except OSError, err:
+            # workaround for missing shebang of epstopdf script
+            if err.errno != getattr(errno, 'ENOEXEC', 0):
+                raise
+            p = subprocess.Popen(['bash'] + args, stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+    except OSError, err:
+        if err.errno != ENOENT:
+            raise
+        raise PlantUmlError('epstopdf command %r cannot be run'
+                            % self.builder.config.plantuml_epstopdf)
+    serr = p.communicate()[1]
+    if p.returncode != 0:
+        raise PlantUmlError('error while running epstopdf\n\n' + serr)
+    return refname[:-4] + '.pdf', fname[:-4] + '.pdf'
+
+_KNOWN_LATEX_FORMATS = {
+    'eps': ('eps', lambda self, refname, fname: (refname, fname)),
+    'pdf': ('eps', _convert_eps_to_pdf),
+    'png': ('png', lambda self, refname, fname: (refname, fname)),
+    }
+
 def latex_visit_plantuml(self, node):
     try:
-        refname, _outfname = render_plantuml(self, node, 'png')
+        format = self.builder.config.plantuml_latex_output_format
+        try:
+            fileformat, postproc = _KNOWN_LATEX_FORMATS[format]
+        except KeyError:
+            raise PlantUmlError(
+                'plantuml_latex_output_format must be one of %s, but is %r'
+                % (', '.join(map(repr, _KNOWN_LATEX_FORMATS)), format))
+        refname, outfname = render_plantuml(self, node, fileformat)
+        refname, outfname = postproc(self, refname, outfname)
     except PlantUmlError, err:
         self.builder.warn(str(err))
         raise nodes.SkipNode
-    self.body.append('\\includegraphics{%s}' % self.encode(refname))
+    self.body.append('\n\\includegraphics{%s}\n' % self.encode(refname))
     raise nodes.SkipNode
 
+def pdf_visit_plantuml(self, node):
+    try:
+        refname, outfname = render_plantuml(self, node, 'eps')
+        refname, outfname = _convert_eps_to_pdf(self, refname, outfname)
+    except PlantUmlError, err:
+        self.builder.warn(str(err))
+        raise nodes.SkipNode
+    rep = nodes.image(uri=outfname, alt=node['alt'] or node['uml'])
+    node.parent.replace(node, rep)
+
 def setup(app):
     app.add_node(plantuml,
                  html=(html_visit_plantuml, None),
@@ -167,3 +218,10 @@ def setup(app):
     app.add_directive('uml', UmlDirective)
     app.add_config_value('plantuml', 'plantuml', 'html')
     app.add_config_value('plantuml_output_format', 'png', 'html')
+    app.add_config_value('plantuml_epstopdf', 'epstopdf', '')
+    app.add_config_value('plantuml_latex_output_format', 'png', '')
+
+    # imitate what app.add_node() does
+    if 'rst2pdf.pdfbuilder' in app.config.extensions:
+        from rst2pdf.pdfbuilder import PDFTranslator as translator
+        setattr(translator, 'visit_' + plantuml.__name__, pdf_visit_plantuml)
